from flask import Flask, render_template
from model.memory import load_dreams
from model.tokenizer import Tokenizer
from context.context import load_context
import os


app = Flask(__name__)
tokenizer = Tokenizer()


def load_loss_data():
    path = "data/logs/loss.log"
    if not os.path.exists(path):
        return []
    with open(path, "r", encoding="utf-8") as f:
        lines = f.readlines()
    return [float(line.strip().split(",")[1]) for line in lines[-50:]]


@app.route("/")
def index():
    dreams = load_dreams()
    top_dreams = dreams[:5]
    memory_size = len(load_context())
    loss_data = load_loss_data()
    return render_template("index.html",
                           vocab_size=len(tokenizer.vocab),
                           top_dreams=top_dreams,
                           memory_size=memory_size,
                           loss_data=loss_data)


def run_dashboard():
    app.run(host="0.0.0.0", port=5000, debug=False, use_reloader=False)
